﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace SmartCart
{
    public partial class SmartCart : Form
    {
        XmlHandler xmlHandler;
        List<Shoplist> shoplists;
        Shoplist productlist;
        List<Label> labelname;
        List<Label> labelamount;
        List<CheckBox> checkbought;
        List<PictureBox> icons;
        Shoplist activeList;
        Configuration configuration;
        String editShoplist;
        String editCategory;
        String editProduct;

        public SmartCart()
        {
            InitializeComponent();
            xmlHandler = new XmlHandler();
            configuration = xmlHandler.XmlToConfiguration();
            shoplists = xmlHandler.XmlToShoplists();
            productlist = xmlHandler.XmlToProductlist();
            xmlHandler.XmlToStatistics();
            PopulateCategoryCombos();
            labelname = new List<Label>();
            labelamount = new List<Label>();
            checkbought = new List<CheckBox>();
            icons = new List<PictureBox>();
            CreateShoppingView();
            CreateOrganizingView();
            PopulateActiveListCombo();
            PopulateStatisticsList();
            // Sätt förvalda värden för val-flikens combo-boxar
            comboLanguage.SelectedItem = configuration.language;
            comboTextSize.SelectedItem = configuration.textSize;
            comboAppearance.SelectedItem = configuration.appearance;
            comboSortBy.SelectedItem = configuration.sortBy;
        }

        private void CreateShoppingView()
        {
            bool found = false;
            foreach (Shoplist shoplist in shoplists)
            {
                if (shoplist.name == configuration.activeList)
                {
                    activeList = shoplist;
                    found = true;
                }
            }
            if (!found)
                activeList = new Shoplist("tmp");
            string size = configuration.textSize;
            int paddingY;

            int PosX = 30;
            int PosY = 5;

            float fontSize;

            if (size == "Liten")
            {
                fontSize = 10;
                paddingY = 16;
            }
            else if (size == "Normal")
            {
                fontSize = 12;
                paddingY = 20;
            }
            else
            {
                fontSize = 14;
                paddingY = 24;
            }

            if (configuration.sortBy == "Bokstav")
                activeList.SortLetter();
            else
                activeList.SortHemkop();

            List<Category> categories = activeList.GetCategories();
            foreach (Label label in labelname)
            {
                tabHandla.Controls.Remove(label);
            }
            foreach (Label label in labelamount)
            {
                tabHandla.Controls.Remove(label);
            }
            foreach (CheckBox checkbox in checkbought)
            {
                tabHandla.Controls.Remove(checkbox);
            }
            foreach (PictureBox icon in icons)
            {
                tabHandla.Controls.Remove(icon);
            }
            labelname.Clear();
            labelamount.Clear();
            checkbought.Clear();
            icons.Clear();

            foreach (Category category in categories)
            {
                string[] tmpcolor = category.color.Split(new char[] { ',' });
                List<int> color = new List<int>();
                foreach (string tcolor in tmpcolor)
                {
                    color.Add(System.Convert.ToInt32(tcolor));
                }
                if (configuration.appearance == "Rubriker")
                {
                    PictureBox catIcon = new PictureBox();
                    catIcon.Image = new System.Drawing.Bitmap(@"\Program Files\SmartCart\icons\" + category.image);
                    catIcon.Size = new System.Drawing.Size(16, 16);
                    catIcon.Location = new System.Drawing.Point(PosX - 25, PosY);
                    catIcon.Visible = true;
                    icons.Add(catIcon);
                    catIcon.Show();

                    Label catLabel = new Label();
                    catLabel.Text = category.name;
                    catLabel.Size = new System.Drawing.Size(140, paddingY);
                    catLabel.Font = new System.Drawing.Font("Tahoma", fontSize, System.Drawing.FontStyle.Bold);
                    catLabel.Location = new System.Drawing.Point(PosX + 15, PosY);
                    catLabel.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(color[0])))), ((int)(((byte)(color[1])))), ((int)(((byte)(color[2])))));
                    catLabel.Visible = true;
                    labelname.Add(catLabel);
                    catLabel.Show();
                    

                    tabHandla.Controls.Add(catIcon);
                    tabHandla.Controls.Add(catLabel);
                    PosY = PosY + paddingY;
                }

                foreach (Product product in category.GetProducts())
                {

                    CheckBox checkbox = new CheckBox();
                    checkbox.Location = new System.Drawing.Point(PosX, PosY);
                    checkbox.Size = new System.Drawing.Size(16, paddingY);
                    checkbox.Name = product.name;
                    checkbox.Tag = category.name;
                    checkbox.Checked = product.bought;
                    checkbought.Add(checkbox);
                    checkbox.Show();

                    Label amountLabel = new Label();
                    amountLabel.Text = product.amount.ToString();
                    amountLabel.Size = new System.Drawing.Size(25, paddingY);
                    amountLabel.TextAlign = System.Drawing.ContentAlignment.TopRight;
                    amountLabel.Font = new System.Drawing.Font("Tahoma", fontSize, System.Drawing.FontStyle.Regular);
                    amountLabel.Location = new System.Drawing.Point(PosX + 25, PosY);

                    Label textLabel = new Label();
                    textLabel.Text = product.name;
                    textLabel.Size = new System.Drawing.Size(140, paddingY);
                    textLabel.Font = new System.Drawing.Font("Tahoma", fontSize, System.Drawing.FontStyle.Bold);
                    textLabel.Location = new System.Drawing.Point(PosX + 55, PosY);
                    if (configuration.appearance == "Ikoner")
                    {
                        amountLabel.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(color[0])))), ((int)(((byte)(color[1])))), ((int)(((byte)(color[2])))));

                        textLabel.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(color[0])))), ((int)(((byte)(color[1])))), ((int)(((byte)(color[2])))));

                        PictureBox icon = new PictureBox();
                        icon.Image = new System.Drawing.Bitmap(@"\Program Files\SmartCart\icons\" + category.image);
                        icon.Size = new System.Drawing.Size(16, 16);
                        icon.Location = new System.Drawing.Point(PosX - 25, PosY);
                        icon.Visible = true;
                        icons.Add(icon);
                        icon.Show();

                        tabHandla.Controls.Add(icon);
                    }

                    amountLabel.Visible = true;
                    labelamount.Add(amountLabel);
                    amountLabel.Show();

                    textLabel.Visible = true;
                    labelname.Add(textLabel);
                    textLabel.Show();

                    tabHandla.Controls.Add(checkbox);
                    tabHandla.Controls.Add(textLabel);
                    tabHandla.Controls.Add(amountLabel);

                    PosY = PosY + paddingY;

                }
            }
        }

        private void CreateOrganizingView()
        {
            // Töm gammal trädvy
            treeOrg.Nodes.Clear();
            int i = 0;
            foreach (Shoplist shoplist in shoplists)
            {
                // Lägg till noder för listor
                treeOrg.Nodes.Add(new TreeNode(shoplist.name));
                TreeNode t1Node = new TreeNode();
                t1Node = treeOrg.Nodes[i];
                int j = 0;
                // Lägg till noder för kategorier
                foreach (Category category in shoplist.GetCategories())
                {
                    t1Node.Nodes.Add(new TreeNode(category.name));
                    TreeNode t2Node = new TreeNode();
                    t2Node = t1Node.Nodes[j];
                    int k = 0;
                    // Lägg till doer för produkter
                    foreach (Product product in category.GetProducts())
                    {
                        t2Node.Nodes.Add(new TreeNode(product.name));
                        TreeNode t3Node = new TreeNode();
                        t3Node = t2Node.Nodes[k];
                        // Lägg till noder för produktegenskaper
                        t3Node.Nodes.Add(new TreeNode("Antal: " + product.amount.ToString()));
                        t3Node.Nodes.Add(new TreeNode("Pris: " + product.price.ToString() + " kr"));
                        k = k +1;
                    }
                    j = j + 1;
                }
                i = i + 1;
            }
        }

        private void CreateEditPanel(string shoplist, string category, string product)
        {
            editShoplist = shoplist;
            editCategory = category;
            editProduct = product;
            foreach (Shoplist tshoplist in shoplists)
            {
                if (tshoplist.name == shoplist)
                {
                    foreach (Category tcategory in tshoplist.GetCategories())
                    {
                        if (tcategory.name == category)
                        {
                            foreach (Product tproduct in tcategory.GetProducts())
                            {
                                if (tproduct.name == product)
                                {
                                    numericEditAmount.Value = tproduct.amount;
                                    numericEditPrice.Value = System.Convert.ToDecimal(tproduct.price);
                                    break;
                                }
                            }
                            break;
                        }
                    }
                    break;
                }
            }               
            buttonEdit.Enabled = false;
            buttonRemove.Enabled = false;
            buttonAdd.Enabled = false;
            panelEditProduct.Show();
        }

        private void PopulateActiveListCombo()
        {
            comboActiveList.Items.Clear();
            foreach (Shoplist shoplist in shoplists)
            {
                comboActiveList.Items.Add(shoplist.name);
            }
            comboActiveList.SelectedItem = activeList.name;
        }

        private void PopulateListCombo()
        {
            comboList.Items.Clear();
            foreach (Shoplist shoplist in shoplists)
            {
                comboList.Items.Add(shoplist.name);
            }
            comboList.SelectedItem = activeList.name;
        }

        private void PopulateCategoryCombos()
        {
            comboCategory.Items.Clear();
            comboCategory2.Items.Clear();
            foreach (Category category in productlist.GetCategories())
            {
                comboCategory.Items.Add(category.name);
                comboCategory2.Items.Add(category.name);
            }
            comboCategory2.SelectedItem = productlist.GetCategories()[0].name;
        }

        private void PopulateProductCombo()
        {
            comboProducts.Items.Clear();
            foreach (Category category in productlist.GetCategories())
            {
                if (category.name == comboCategory.SelectedItem.ToString())
                {
                    foreach (Product product in category.GetProducts())
                    {
                        comboProducts.Items.Add(product.name);
                    }
                }
            }
        }

        private void PopulateStatisticsList()
        {
            listStatistics.Items.Clear();
            foreach (string year in Statistics.years.Keys)
            {
                listStatistics.Items.Add("Januari " + year + " = " + Statistics.years[year]["1"]);
                listStatistics.Items.Add("Februari " + year + " = " + Statistics.years[year]["2"]);
                listStatistics.Items.Add("Mars " + year + " = " + Statistics.years[year]["3"]);
                listStatistics.Items.Add("April " + year + " = " + Statistics.years[year]["4"]);
                listStatistics.Items.Add("Maj " + year + " = " + Statistics.years[year]["5"]);
                listStatistics.Items.Add("Juni " + year + " = " + Statistics.years[year]["6"]);
                listStatistics.Items.Add("Juli " + year + " = " + Statistics.years[year]["7"]);
                listStatistics.Items.Add("Augusti " + year + " = " + Statistics.years[year]["8"]);
                listStatistics.Items.Add("September " + year + " = " + Statistics.years[year]["9"]);
                listStatistics.Items.Add("Oktober " + year + " = " + Statistics.years[year]["10"]);
                listStatistics.Items.Add("November " + year + " = " + Statistics.years[year]["11"]);
                listStatistics.Items.Add("December " + year + " = " + Statistics.years[year]["12"]);
            }
        }

        private void SmartCart_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == System.Windows.Forms.Keys.F1))
            {
                Modes.SelectedIndex = 0;
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F2))
            {
                Modes.SelectedIndex = 2;
            }
        }

        private int NodeLevel(TreeNode node)
        {
            int level = 0;
            while ((node = node.Parent) != null) ++level;
            return level;
        }

        private void AddShoplist(String shoplist)
        {
            bool found = false;
            foreach (Shoplist tshoplist in shoplists)
            {
                if (tshoplist.name == shoplist)
                    found = true;
            }
            if (!found)
            {
                Shoplist nshoplist = new Shoplist(shoplist);
                if (shoplists.Count() == 0)
                {
                    configuration.activeList = nshoplist.name;
                    activeList = nshoplist;
                    PopulateActiveListCombo();
                    PopulateListCombo();
                }
                shoplists.Add(nshoplist);
            }
        }

        private void AddProduct(String shoplist, String category, String product, String amount)
        {
            foreach (Category tcategory in productlist.GetCategories())
            {
                if (tcategory.name == category)
                {
                    foreach (Product tproduct in tcategory.GetProducts())
                    {
                        if (tproduct.name == product)
                        {
                            foreach (Shoplist rshoplist in shoplists)
                            {
                                if (rshoplist.name == shoplist)
                                {
                                    bool found = false;
                                    foreach (Category rcategory in rshoplist.GetCategories())
                                    {
                                        if (rcategory.name == category)
                                        {
                                            found = true;
                                            break;
                                        }
                                    }
                                    if (!found)
                                    {
                                        Category scategory = new Category(tcategory.name, tcategory.color, tcategory.position.ToString(), tcategory.image);
                                        rshoplist.AddCategory(scategory);
                                    }
                                    found = false;
                                    foreach (Category rcategory in rshoplist.GetCategories())
                                    {
                                        if (rcategory.name == category)
                                        {
                                            foreach (Product rproduct in rcategory.GetProducts())
                                            {
                                                if (rproduct.name == product)
                                                {
                                                    found = true;
                                                    break;
                                                }
                                            }
                                            break;
                                        }
                                    }
                                    if (!found)
                                    {
                                        foreach (Category rcategory in rshoplist.GetCategories())
                                        {
                                            if (rcategory.name == category)
                                            {
                                                rcategory.AddProduct(tproduct);
                                                foreach (Product rproduct in rcategory.GetProducts())
                                                {
                                                    if (rproduct.name == product)
                                                    {
                                                        rproduct.amount = System.Convert.ToInt32(amount);
                                                    }
                                                }
                                                break;
                                            }
                                        }
                                    }
                                    break;
                                }
                                
                            }
                            break;
                        }
                    }
                    break;
                }
            }
        }

        private void EditProduct(String amount, String price)
        {
            foreach (Shoplist shoplist in shoplists)
            {
                if (shoplist.name == editShoplist)
                {
                    foreach (Category category in shoplist.GetCategories())
                    {
                        if (category.name == editCategory)
                        {
                            foreach (Product product in category.GetProducts())
                            {
                                if (product.name == editProduct)
                                {
                                    product.amount = System.Convert.ToInt32(amount);
                                    product.price = System.Convert.ToInt32(price);
                                    break;
                                }
                            }
                            break;
                        }
                    }
                    break;
                }
            }
            editShoplist = null;
            editCategory = null;
            editProduct = null;
        }

        private void DeleteProduct(String shoplist, String category, String product)
        {
            Product rproduct = new Product("tmp", "0", "false", "0");
            foreach (Shoplist tshoplist in shoplists)
            {
                if (tshoplist.name == shoplist)
                {
                    foreach (Category tcategory in tshoplist.GetCategories())
                    {
                        if (tcategory.name == category)
                        {
                            foreach (Product tproduct in tcategory.GetProducts())
                            {
                                if (tproduct.name == product)
                                {
                                    rproduct = tproduct;
                                    break;
                                }
                            }
                            tcategory.DeleteProduct(rproduct);
                            break;
                        }
                    }
                    break;
                }
            }
        }

        private void DeleteCategory(String shoplist, String category)
        {
            Category rcategory = new Category("tmp", "0,0,0", "0", "tmp");
            foreach (Shoplist tshoplist in shoplists)
            {
                if (tshoplist.name == shoplist)
                {
                    foreach (Category tcategory in tshoplist.GetCategories())
                    {
                        if (tcategory.name == category)
                        {
                            rcategory = tcategory;
                            break;
                        }
                    }
                    tshoplist.DeleteCategory(rcategory);
                    break;
                }
            }
        }

        private void DeleteShoplist(String shoplist)
        {
            Shoplist rshoplist = new Shoplist("tmp");
            foreach (Shoplist tshoplist in shoplists)
            {
                if (tshoplist.name == shoplist)
                {
                    rshoplist = tshoplist;
                    break;
                }
            }
            shoplists.Remove(rshoplist);
            if (shoplists.Count > 0)
                configuration.activeList = shoplists[0].name;
            else
                configuration.activeList = "";
        }

        private void AddNewProduct(String category, String product)
        {
            foreach (Category tcategory in productlist.GetCategories())
            {
                if (tcategory.name == category)
                {
                    bool found = false;
                    foreach (Product tproduct in tcategory.GetProducts())
                    {
                        if (tproduct.name == product)
                        {
                            found = true;
                            break;
                        }
                    }
                    if (!found)
                    {
                        Product newProduct = new Product(product, "0", "false", "0");
                        tcategory.AddProduct(newProduct);
                    }
                    break;
                }
            }
            PopulateProductCombo();
        }

        private void CheckBought()
        {
            Statistics.checkStatYear();
            foreach (CheckBox checkbox in checkbought)
            {
                if (checkbox.Checked == true)
                {
                    foreach (Category category in activeList.GetCategories())
                    {
                        if (category.name == checkbox.Tag.ToString())
                        {
                            foreach (Product product in category.GetProducts())
                            {
                                if (checkbox.Name == product.name)
                                {
                                    if (product.bought == false)
                                        Statistics.AddValue(product.price * product.amount);
                                    product.bought = true;
                                }
                            }
                        }
                    }
                }
                else
                {
                    foreach (Category category in activeList.GetCategories())
                    {
                        if (category.name == checkbox.Tag.ToString())
                        {
                            foreach (Product product in category.GetProducts())
                            {
                                if (checkbox.Name == product.name)
                                {
                                    if (product.bought)
                                        Statistics.AddValue(-(product.price * product.amount));
                                    product.bought = false;
                                }
                            }
                        }
                    }
                }
            }
        }

        private void treeOrg_GotFocus(object sender, EventArgs e)
        {
            buttonRemove.Enabled = true;
            buttonEdit.Enabled = true;
        }

        private void Modes_GotFocus(object sender, EventArgs e)
        {
            buttonRemove.Enabled = false;
            buttonEdit.Enabled = false;
            PopulateActiveListCombo();
            CheckBought();
            CreateShoppingView();
            PopulateStatisticsList();
        }

        private void buttonEdit_Click(object sender, EventArgs e)
        {
            // hämta node
            TreeNode selectedNode = new TreeNode();
            selectedNode = treeOrg.SelectedNode;
            if (selectedNode != null)
            {
                int nodeLevel = NodeLevel(selectedNode);
                if (nodeLevel == 2)
                {
                    CreateEditPanel(selectedNode.Parent.Parent.Text, selectedNode.Parent.Text, selectedNode.Text);
                }
                else if (nodeLevel == 3)
                {
                    CreateEditPanel(selectedNode.Parent.Parent.Parent.Text, selectedNode.Parent.Parent.Text, selectedNode.Parent.Text);
                }
            }
        }

        private void buttonRemove_Click(object sender, EventArgs e)
        {
            // hämta node
            TreeNode selectedNode = new TreeNode();
            selectedNode = treeOrg.SelectedNode;
            if (selectedNode != null)
            {
                int nodeLevel = NodeLevel(selectedNode);
                bool remove = false;
                if (nodeLevel == 2)
                {
                    DeleteProduct(selectedNode.Parent.Parent.Text, selectedNode.Parent.Text, selectedNode.Text);
                    remove = true;
                }
                else if (nodeLevel == 1)
                {
                    DeleteCategory(selectedNode.Parent.Text, selectedNode.Text);
                    remove = true;
                }
                else if (nodeLevel == 0)
                {
                    DeleteShoplist(selectedNode.Text);
                    remove = true;
                }
                if (remove)
                    selectedNode.Remove();
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            buttonEdit.Enabled = true;
            buttonAdd.Enabled = true;
            buttonRemove.Enabled = true;
            AddPanel.Hide();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            if (comboList.SelectedItem != null && comboCategory.SelectedItem != null && comboProducts.SelectedItem != null)
            {
                AddProduct(comboList.SelectedItem.ToString(), comboCategory.SelectedItem.ToString(), comboProducts.SelectedItem.ToString(), numericUpDown.Value.ToString());
                CreateOrganizingView();
                buttonEdit.Enabled = true;
                buttonAdd.Enabled = true;
                buttonRemove.Enabled = true;
                AddPanel.Hide();
            }
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            PopulateListCombo();
            buttonEdit.Enabled = false;
            buttonAdd.Enabled = false;
            buttonRemove.Enabled = false;
            AddPanel.Show();
        }

        private void buttonAddList_Click(object sender, EventArgs e)
        {
            panelAddList.Show();
            panel2.Enabled = false;
        }

        private void buttonListCancel_Click(object sender, EventArgs e)
        {
            panelAddList.Hide();
            panel2.Enabled = true;
        }

        private void buttonListAdd_Click(object sender, EventArgs e)
        {
            if (textBoxList.Text.ToString().Length > 0)
            {
                AddShoplist(textBoxList.Text.ToString());
                PopulateListCombo();
                panelAddList.Hide();
                panel2.Enabled = true;
                CreateOrganizingView();
            }
        }

        private void buttonAddProduct_Click(object sender, EventArgs e)
        {
            panelAddProduct.Show();
            panel2.Enabled = false;
        }

        private void buttonProductCancel_Click(object sender, EventArgs e)
        {
            panelAddProduct.Hide();
            panel2.Enabled = true;
        }

        private void buttonProductAdd_Click(object sender, EventArgs e)
        {
            if (comboCategory2.SelectedItem != null && textBoxProduct.Text.ToString().Length > 0)
            {
                AddNewProduct(comboCategory2.SelectedItem.ToString(), textBoxProduct.Text.ToString());
                panelAddProduct.Hide();
                panel2.Enabled = true;
            }
        }

        private void comboCategory_SelectedIndexChanged(object sender, EventArgs e)
        {
            PopulateProductCombo();
        }

        private void comboActiveList_SelectedIndexChanged(object sender, EventArgs e)
        {
            foreach (Shoplist shoplist in shoplists)
            {
                if (shoplist.name == comboActiveList.SelectedItem.ToString())
                {
                    activeList = shoplist;
                    configuration.activeList = comboActiveList.SelectedItem.ToString();
                }
            }
        }

        private void comboLanguage_SelectedIndexChanged(object sender, EventArgs e)
        {
            configuration.language = comboLanguage.SelectedItem.ToString();
        }

        private void comboTextSize_SelectedIndexChanged(object sender, EventArgs e)
        {
            configuration.textSize = comboTextSize.SelectedItem.ToString();
        }

        private void comboAppearance_SelectedIndexChanged(object sender, EventArgs e)
        {
            configuration.appearance = comboAppearance.SelectedItem.ToString();
        }

        private void comboSortBy_SelectedIndexChanged(object sender, EventArgs e)
        {
            configuration.sortBy = comboSortBy.SelectedItem.ToString();
        }

        private void buttonEditCancel_Click(object sender, EventArgs e)
        {
            buttonEdit.Enabled = true;
            buttonAdd.Enabled = true;
            buttonRemove.Enabled = true;
            panelEditProduct.Hide();
        }

        private void buttonEditEdit_Click(object sender, EventArgs e)
        {
            EditProduct(numericEditAmount.Value.ToString(), numericEditPrice.Value.ToString());
            buttonEdit.Enabled = true;
            buttonAdd.Enabled = true;
            buttonRemove.Enabled = true;
            CreateOrganizingView();
            panelEditProduct.Hide();
        }

        private void SmartCart_Closing(object sender, CancelEventArgs e)
        {
            xmlHandler.ConfigurationToXml(configuration);
            xmlHandler.ShoplistsToXml(shoplists);
            xmlHandler.StatisticsToXml();
            xmlHandler.ProductlistToXml(productlist);
        }
    }
}